# -*- coding: utf-8 -*-
#
# Improvements compared to original diagram
# - linspace instead of range includes endpoint
# - spaces
# - our material properties
# - changed shading of diffusive convection area


import math as m
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import pylab

from matplotlib.ticker import ScalarFormatter, FormatStrFormatter

# fonts and sizes
params = {'font.family' : 'Nimbus Roman', 'mathtext.fontset' : 'custom', 'mathtext.it' : 'STIXGeneral:italic', 'mathtext.fontset' : 'stix', 'mathtext.bf' : 'STIXGeneral:bold'}
pylab.rcParams.update(params)
plt.rcParams.update({'font.size': 16, 'lines.linewidth':2})
plt.rcParams['figure.figsize'] = [6, 4] # keep width 6, but you may change height (4)

#plt.ticklabel_format(scilimits=(-1e20,1e20))

# definition of HZDR-colours
hzdrBlue = '#005AA0'
blue80 = '#2473AD'
blue60 = '#6691C2'
blue40 = '#9BB3D7'
blue20 = '#CDD8EC'

hzdrOrange = '#F0781E'
orange80 = '#F39150'
orange60 = '#F7AF7C'
orange40 = '#FBCBA8'
orange20 = '#FDE6D4'

gray1 = '#515151'
gray2 = '#9C9C9C'
gray3 = '#B9B9B9'

# secondary colours (use only if really necessary)
hzdrDarkBlue = '#003E6E'
hzdrRed = '#D42D12'
hzdrYellow = '#E6AF11'
hzdrGreen = '#144D28'
hzdrLightGreen = '#A9B509'
hzdrLightBlue = '#00A2E0'



# the usual parameters are:
# - thermal Rayleigh nb 
# - Prandtl number (sigma)
# - diffusivity ratio (tau)
# - density ratio (R) (or solutal Rayleigh nb)

# **********************************************************************
# parameters
# **********************************************************************

RaMax = 2.2e11
RsMax = 2.2e11

# Rayleigh nb over full range
Ra = np.linspace(-RaMax, RaMax, 100)

# Solutal Rayleigh number over full range
Rs = np.linspace(-RsMax, RsMax, 100)

# diffusivity ratio according to Turner1974: salt/temperature (inverse to other authors!!)
# tau = 0.01 # Radko2013, p.13
tau = 0.003 # 1/Lewis 0.003 and 0.004 for our salts

# Prandtl number 
#Pr = 7. # Radko2013, p.13
Pr = 7.1 # 7.1 and 5.7 for our salt


# **********************************************************************
# Turner1974: Double-diffusive phenomena
# Turner1973: Buoyancy effects in fluids
# **********************************************************************


# **********************************************************************
# different functions
# **********************************************************************

# Turner 1974, eqn 2
def XZ(Rs,Pr,tau):
 return Rs/tau + 27.0*m.pi**4 / 4.0

# Turner 1974, eqn 3
def XW(Rs,Pr,tau):
 return ((Pr + tau)/(Pr + 1.0))*Rs + (1.0 + tau)*(1.0 + (tau/Pr))*(27.0*m.pi**4/4.0)

# **********************************************************************
# Point X where two lines meet
# **********************************************************************

# formula (2) == (3), page 39, Turner1974
RsCr = ((1.0 + tau)*(1.0 + (tau/Pr))*(27.0*m.pi**4/4.0) - 27.0*m.pi**4 / 4.0) / (1.0/tau - (Pr + tau)/(Pr + 1.0))
print('Rs cr:', RsCr)

# line X-U

def XU(Rs,Pr,tau):
 slope = XW(RsCr,Pr,tau) / RsCr
 return slope*Rs


# **********************************************************************
# plot
# **********************************************************************

fig, ax = plt.subplots()

# line X-Z
RsLeft = np.linspace(-RsMax, RsCr, 100)
ax.plot(RsLeft, XZ(RsLeft,Pr,tau), '-', color='k', linewidth=0.5)

# line X-W
RsRight = np.linspace(RsCr, RsMax, 100)
ax.plot(RsRight, XW(RsRight,Pr,tau), '-', color='k', linewidth=0.5)

# line X-U
ax.plot(RsRight, XU(RsRight,Pr,tau), '-', color='k', linewidth=0.5)

# diagonal line (unit slope)
ax.plot(Rs, Rs, dashes=[20, 10, 20, 10], color='k', linewidth=0.5)

# Tilgner line
ax.plot(RsLeft, RsLeft/100., dashes=[20, 10, 20, 10], color='k', linewidth=0.5) # Hage & Tilgner 2010
ax.plot(RsLeft, RsLeft/30., dashes=[20, 10, 20, 10], color='k', linewidth=0.5) # Kellner & Tilgner 2014

# coordinate axis
ax.plot([-RsMax,RsMax], [0,0], color='k')
ax.plot([0,0], [-RaMax,RaMax], color='k')

# shading
plt.fill_between(RsLeft, RsLeft/100., RsLeft/30., color=gray3, lw=0)			# Tilgner area 1
plt.fill_between(RsLeft, RsLeft/30., RsLeft, color=gray3, lw=0)         		# Tilgner area 2
plt.fill_between(RsLeft, RsLeft, XZ(RsLeft,Pr,tau), color=gray2, lw=0)         		# classic fingers

#plt.fill_between(RsRight, XU(RsRight,Pr,tau), XW(RsRight,Pr,tau), color=grau1,linewidth=0)         # diffusive convection
plt.fill_between(RsRight, RaMax, XW(RsRight,Pr,tau), color=gray2, lw=0)         	# diffusive convection

#plt.fill_between(RsLeft, XZ(RsLeft,Pr,tau), -RaMax, color='w', linewidth=0)		# white below fingers
plt.fill_between(RsRight, RsRight, -RaMax, color='w', lw=0)         			# white below diffusive convection

#RsRightComplete = np.linspace(0, RsMax, 10)						# remove some thin gray line
#plt.fill_between(RsRightComplete, XW(RsRightComplete,Pr,tau), -RaMax, color='w', linewidth=0)



# plot ellipse for upper layer at discharge
# 1 is smallest, 2 is largest value
Ra1 = 6.9e+02
Ra2 = 3.6e+07
Rs1 = 8.2e+07			# sign of Rs had to be inversed to fit to Turner diagram
Rs2 = 2.1e+11

#Ra = 1.2e7
#Rs = 8.4e10

deltaRa = Ra2-Ra1            	# difference in Ra numbers
deltaRs = Rs2-Rs1

Ram = (Ra1 + deltaRa/2.0) 	# mean Ra
Rsm = (Rs1 + deltaRs/2.0) 	# mean Rs

#ellipse = Ellipse((Rsm,Ram), deltaRs, deltaRa, linewidth=0.5, fill=True, facecolor=rossendorfOrange)
#ax.add_artist(ellipse)
ax.plot([Rs1+deltaRs], [Ra1+deltaRa], 'o', markersize = 8., color=hzdrBlue, label='upper layer')
print('Discharge of upper layer: Ra (mean):', Ram, ' and Rs (mean):', Rsm)



# plot ellipse for lower layer at discharge
# 1 is smallest, 2 is largest value
Ra1 = -7.0e+02
Ra2 = -3.5e+07
Rs1 = -1.2e+07			# sign of Rs had to be inversed to fit to diagram
Rs2 = -3.0e+10
   
#Ra = -1.1e7
#Rs = -1.2e10

deltaRa = Ra2-Ra1            	# difference in Ra numbers
deltaRs = Rs2-Rs1

Ram = (Ra1 + deltaRa/2.0) 	# mean Ra
Rsm = (Rs1 + deltaRs/2.0) 	# mean Rs

#ellipse = Ellipse((Rsm,Ram), deltaRs, deltaRa, linewidth=0.5, fill=True, facecolor=rossendorfOrange)
#ax.add_artist(ellipse)
ax.plot([Rs1+deltaRs], [Ra1+deltaRa], 'o', markersize = 8., color=hzdrOrange, label='lower layer')
print([Rs1+deltaRs])
print([Ra1+deltaRa])
print('Discharge of lower layer: Ra (mean):', Ram, ' and Rs (mean):', Rsm)



# stable, unstable regions
ax.text( 0.5*RsMax,  -0.5*RaMax, 'no flow\n\n(stable)', multialignment='center', horizontalalignment='center', verticalalignment='center')
ax.text(-0.5*RsMax,   0.3*RaMax, 'Rayleigh-Bénard-like\nconvection (unstable)', multialignment='center', horizontalalignment='center', verticalalignment='center')
ax.text(-0.4*RsMax,  -0.8*RaMax, 'finger\nconvection', multialignment='center', horizontalalignment='center', verticalalignment='center')
ax.text(-0.65*RsMax, -0.2*RaMax, 'possibly\nfinger convection', multialignment='center', horizontalalignment='center', verticalalignment='center')
ax.text(0.45*RsMax,   0.9*RaMax, 'diffusive convection', multialignment='center', horizontalalignment='center', verticalalignment='center')


plt.xlim(-RsMax, RsMax)
plt.ylim(-RaMax, RaMax)
#ax.set_xticks([], [])
#ax.set_yticks([], [])

plt.legend()

ax.set_xlabel(r'$-Ra_\mathrm{c}$')
ax.set_ylabel(r'$Ra_\mathrm{T}$')

#ax.set_xticklabels(['a', r'$-2\times10^{-11}$', r'$-10^{-11}$', '0', r'$10^{-11}$', r'$2\times10^{-11}$'])
#ax.set_yticklabels(['a', r'$-2\times10^{-11}$', r'$-10^{-11}$', '0', r'$10^{-11}$', r'$2\times10^{-11}$'])

fig.subplots_adjust(left=.13,bottom=.14,right=.96,top=.94,wspace=.2,hspace=.0)
fig.savefig("TurnerDiagram.pdf")
