import numpy as np
import matplotlib.pyplot as plt


# material properties
lambda_c    = 60	                # thermal conductivity heavy metal (bottom/cathode)
lambda_sC   = 0.084	                # thermal conductivity heavy salt (bottom/salt)
lambda_sA   = 0.84	                # thermal conductivity light salt (top/salt)
lambda_a    = 63	                # thermal conductivity light metal (top/anode)

h_c         = 8e-3                      # height cathode
h_sC        = 20e-3                     # height salt on cathode side
h_sA        = 20e-3                     # height salt on anode side
h_a         = 13e-3                     # height anode

# geometry
h1          = h_c	            # height interface cathode-salt
h2          = h_c+h_sC;             # height interface salt-salt
h3          = h_c+h_sC+h_sA;        # height interface salt-anode
H           = h_c+h_sC+h_sA+h_a;    # height of the cell

T_ref       = 600 + 273.15	        # reference temperature

j           = 0.1e4	    		# current density in A/m²
sigma_sC    = 92;                   	# electric conductivity lower salt
sigma_sA    = 138;			# electric conductivity upper salt


###############################################################################
# SIMPLE PURE CONDUCTION

# HEAT GENERATION
qDot_sC =  j**2/sigma_sC;
qDot_sA =  j**2/sigma_sA;

# ANALYTICAL MODEL
# constants
L1 = lambda_a*lambda_sA*lambda_sC;
L2 = lambda_a*lambda_c*lambda_sA;
L3 = lambda_a*lambda_c*lambda_sC;
L4 = lambda_c*lambda_sA*lambda_sC;

C5 = (
     h3*h_a*L4*qDot_sA - h1*h_sC*L1*qDot_sC + h2*h_sC*L2*qDot_sA
     + h1*h2*(L2*qDot_sC + L1*qDot_sA)
     + (h3**2-h2**2)*L3*qDot_sA/2 - (h1**2+h2**2)*L2*qDot_sC/2
     )\
     /\
     (lambda_sA* ( h_c*L1 + h_sC*L2 + h_sA*L3 + h_a*L4 ));
C05 = (H*h3*lambda_c*lambda_sA*lambda_sC*qDot_sA - h1**2*lambda_a*lambda_c*lambda_sA*qDot_sC/2 + h1**2*lambda_a*lambda_sA*lambda_sC*qDot_sC - h1*h2*lambda_a*lambda_c*lambda_sA*qDot_sA + h1*h2*lambda_a*lambda_c*lambda_sA*qDot_sC + h1*h2*lambda_a*lambda_sA*lambda_sC*qDot_sA - h1*h2*lambda_a*lambda_sA*lambda_sC*qDot_sC + h2**2*lambda_a*lambda_c*lambda_sA*qDot_sA - h2**2*lambda_a*lambda_c*lambda_sA*qDot_sC/2 - h2**2*lambda_a*lambda_c*lambda_sC*qDot_sA/2 + h3**2*lambda_a*lambda_c*lambda_sC*qDot_sA/2 - h3**2*lambda_c*lambda_sA*lambda_sC*qDot_sA)/(lambda_sA*(H*lambda_c*lambda_sA*lambda_sC - h1*lambda_a*lambda_c*lambda_sA + h1*lambda_a*lambda_sA*lambda_sC + h2*lambda_a*lambda_c*lambda_sA - h2*lambda_a*lambda_c*lambda_sC + h3*lambda_a*lambda_c*lambda_sC - h3*lambda_c*lambda_sA*lambda_sC))
C3 = (qDot_sC-qDot_sA) * h2 / lambda_sC + (lambda_sA/lambda_sC) * C5;
C1 = -(qDot_sC/lambda_c) * h1 + (lambda_sC/lambda_c) * C3;
C2 = T_ref
C4 = (qDot_sC/2/lambda_sC) * h1**2 + (C1 - C3) * h1 + T_ref;
C6 = (qDot_sA/2/lambda_sA - qDot_sC/2/lambda_sC) * h2**2 + (C3 - C5) * h2 + C4;
C7 = -(qDot_sA/lambda_a) * h3 + (lambda_sA/lambda_a) * C5;
C8 = T_ref - C7 * H;


# temperature distribution
def T_c(z) : # cathode
    TTT = (C1*z+C2);
    return TTT
def T_sC(z) : # lower salt
    TTT = -qDot_sC/2/lambda_sC * z**2 + C3 * z + C4;
    return TTT
def T_sA(z) : # upper salt
    TTT = -qDot_sA/2/lambda_sA * z**2 + C5 * z + C6;
    return TTT
def T_a(z) : # anode
    TTT = (C7*z+C8);
    return TTT


# define figure
f = plt.figure(1)
ax = f.subplots()
#f.set_size_inches(1.5,3)

ax.xaxis.label.set_size(12)
ax.yaxis.label.set_size(12)
ax.tick_params(axis="x", labelsize=10)
ax.tick_params(axis="y", labelsize=10)

# z - For the Plot 
z_c = np.linspace(h1-h1,h1,1000);
z_sC = np.linspace(h1,h2,1000);
z_sA = np.linspace(h2,h3,1000);
z_a = np.linspace(h3,H,1000);

T_PC_c = T_c(z_c)-T_ref
T_PC_sC = T_sC(z_sC)-T_ref
T_PC_sA = T_sA(z_sA)-T_ref
T_PC_a = T_a(z_a)-T_ref


# Plot (T,z)
ax.plot(T_PC_c,  z_c*1e3,  linewidth=4, color='k')
ax.plot(T_PC_sC, z_sC*1e3, linewidth=4, color='k')
ax.plot(T_PC_sA, z_sA*1e3, linewidth=4, color='k')
ax.plot(T_PC_a,  z_a*1e3,  linewidth=4, color='k')

plt.title('analytical solution')
plt.ylim( (0, H*1e3) )
#plt.xlim(0,0.4)

ax.set(xlabel=r"$\Delta T$ in K", ylabel=r"$z$ in mm")
#f.subplots_adjust(left=0.35,bottom=0.15,right=0.922,top=0.97,wspace=0.05,hspace=0.08)

f.savefig('f.pdf')
