#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Steady state pure heat conduction in four layers Na||Zn cell 
Numerical approach
Finite difference only space - second order accurate
"""

import numpy as np
import matplotlib.pyplot as plt


def generate_sub_matrix_single_layer(
    internal_heat_generation,
    layer_origin,
    layer_height,
    number_of_points,
    thermal_conductivity,
):
    interspace = layer_height / (number_of_points - 1)
    if layer_origin == 0.0:
        space_discretisation = np.linspace(
            layer_origin, layer_origin + layer_height, number_of_points
        )
        subdiagonal = (thermal_conductivity / interspace**2) * np.ones(
            number_of_points - 1
        )
    else:
        space_discretisation = np.linspace(
            layer_origin + interspace, layer_origin + layer_height, number_of_points
        )
        subdiagonal = (thermal_conductivity / interspace**2) * np.ones(
            number_of_points
        )
    diagonal = (-2 * thermal_conductivity / interspace**2) * np.ones(number_of_points)
    b_column = -internal_heat_generation * np.ones(number_of_points)  # Ax = b
    return b_column, diagonal, interspace, subdiagonal, space_discretisation


# Geometry
print("WARNING: Update the geometry with real values!")
# I take the lab cell
layer_origin_P = 0.0
layer_height_P = 8e-3  # height of positive electrode - zinc
layer_origin_Eb = layer_origin_P + layer_height_P
layer_height_Eb = 20e-3  # height of electrolyte bottom
layer_origin_Et = layer_origin_Eb + layer_height_Eb
layer_height_Et = 20e-3  # height of electrolyte top
layer_origin_N = layer_origin_Et + layer_height_Et
layer_height_N = 13e-3  # height of negative electrode - sodium
full_cell_height = layer_height_N + layer_height_Et + layer_height_Eb + layer_height_P

# Properties
print("WARNING: Update the properties with real values!")
# I use values of the charged cell, i.e. two different salts!
# Positive electrode - zinc
thermal_conductivity_P = 60
electrical_resistivity_P = 1./2.8e6
# Bottom electrolyte
thermal_conductivity_Eb = 0.084
electrical_resistivity_Eb = 1./92
# Top electrolyte
thermal_conductivity_Et = 0.84
electrical_resistivity_Et = 1./138
# Negative electrode - sodium
thermal_conductivity_N = 63
electrical_resistivity_N = 1./3.1e6

# Boundary Condition
basis_temperature = 0  # Dirichlet BC on the basis
initial_temperature = 600+273.15
# Homogeneus Neumann BC on the lateral side

# Internal heat generation
current_density = 0.1e4
internal_heat_generation_P = electrical_resistivity_P * current_density**2
internal_heat_generation_Eb = electrical_resistivity_Eb * current_density**2
internal_heat_generation_Et = electrical_resistivity_Et * current_density**2
internal_heat_generation_N = electrical_resistivity_N * current_density**2

# Discretisation
number_of_points_P = 1001
number_of_points_Eb = 1001
number_of_points_Et = 1001
number_of_points_N = 1001

# Build single layer matrix
(
    b_column_P,
    diagonal_P,
    interspace_P,
    subdiagonal_P,
    space_discretisation_P,
) = generate_sub_matrix_single_layer(
    internal_heat_generation_P,
    layer_origin_P,
    layer_height_P,
    number_of_points_P,
    thermal_conductivity_P,
)
(
    b_column_Eb,
    diagonal_Eb,
    interspace_Eb,
    subdiagonal_Eb,
    space_discretisation_Eb,
) = generate_sub_matrix_single_layer(
    internal_heat_generation_Eb,
    layer_origin_Eb,
    layer_height_Eb,
    number_of_points_Eb,
    thermal_conductivity_Eb,
)
(
    b_column_Et,
    diagonal_Et,
    interspace_Et,
    subdiagonal_Et,
    space_discretisation_Et,
) = generate_sub_matrix_single_layer(
    internal_heat_generation_Et,
    layer_origin_Et,
    layer_height_Et,
    number_of_points_Et,
    thermal_conductivity_Et,
)
(
    b_column_N,
    diagonal_N,
    interspace_N,
    subdiagonal_N,
    space_discretisation_N,
) = generate_sub_matrix_single_layer(
    internal_heat_generation_N,
    layer_origin_N,
    layer_height_N,
    number_of_points_N,
    thermal_conductivity_N,
)

# Build the full matrix for the solution
full_space_discretisation = np.concatenate(
    (
        space_discretisation_P,
        space_discretisation_Eb,
        space_discretisation_Et,
        space_discretisation_N,
    )
)
full_number_of_points = len(full_space_discretisation)
full_diagonal = np.concatenate(
    (
        diagonal_P,
        diagonal_Eb,
        diagonal_Et,
        diagonal_N,
    )
)
full_subdiagonal = np.concatenate(
    (
        subdiagonal_P,
        subdiagonal_Eb,
        subdiagonal_Et,
        subdiagonal_N,
    )
)
full_b_column = np.concatenate(
    (
        b_column_P,
        b_column_Eb,
        b_column_Et,
        b_column_N,
    )
)
full_A_matrix = (
    np.diagflat(full_diagonal)
    + np.diagflat(full_subdiagonal, -1)
    + np.diagflat(full_subdiagonal, 1)
)
end = full_number_of_points - 1

# Bottom boundary conditions
full_A_matrix[0, 0] = 1.0
full_A_matrix[0, 1] = 0.0
full_b_column[0] = basis_temperature

# Top boundary conditions
full_A_matrix[end, end - 1] = 0.0
full_A_matrix[end, end] = 1.0
full_b_column[end] = basis_temperature

# Internal boundary condition - positive electrode - bottom electrolyte
full_A_matrix[number_of_points_P - 1, number_of_points_P - 2] = (
    -thermal_conductivity_P / interspace_P
)
full_A_matrix[number_of_points_P - 1, number_of_points_P - 1] = (
    thermal_conductivity_P / interspace_P
) + (thermal_conductivity_Eb / interspace_Eb)
full_A_matrix[number_of_points_P - 1, number_of_points_P] = (
    -thermal_conductivity_Eb / interspace_Eb
)
full_b_column[number_of_points_P - 1] = 0.0

# Internal boundary condition - bottom electrolyte - top electrolyte
full_A_matrix[
    number_of_points_P + number_of_points_Eb - 1,
    number_of_points_P + number_of_points_Eb - 2,
] = (
    -thermal_conductivity_Eb / interspace_Eb
)
full_A_matrix[
    number_of_points_P + number_of_points_Eb - 1,
    number_of_points_P + number_of_points_Eb - 1,
] = (thermal_conductivity_Eb / interspace_Eb) + (
    thermal_conductivity_Et / interspace_Et
)
full_A_matrix[
    number_of_points_P + number_of_points_Eb - 1,
    number_of_points_P + number_of_points_Eb,
] = (
    -thermal_conductivity_Et / interspace_Et
)
full_b_column[number_of_points_P + number_of_points_Eb - 1] = 0.0

# Internal boundary condition -  top electrolyte - negative electrode
full_A_matrix[
    number_of_points_P + number_of_points_Eb + number_of_points_Et - 1,
    number_of_points_P + number_of_points_Eb + number_of_points_Et - 2,
] = (
    -thermal_conductivity_Et / interspace_Et
)
full_A_matrix[
    number_of_points_P + number_of_points_Eb + number_of_points_Et - 1,
    number_of_points_P + number_of_points_Eb + number_of_points_Et - 1,
] = (thermal_conductivity_Et / interspace_Et) + (thermal_conductivity_N / interspace_N)
full_A_matrix[
    number_of_points_P + number_of_points_Eb + number_of_points_Et - 1,
    number_of_points_P + number_of_points_Eb + number_of_points_Et,
] = (
    -thermal_conductivity_N / interspace_N
)
full_b_column[number_of_points_P + number_of_points_Eb + number_of_points_Et - 1] = 0.0

# Solve the linear system of equations
temperature_profile = np.linalg.solve(full_A_matrix, full_b_column)

# Plot the temperature profile 
plt.figure(1)
plt.plot(temperature_profile, full_space_discretisation * 1e3)
plt.xlabel("$T$ in K")
plt.ylabel("$z$ in mm")
plt.ylim( 0, 61)
plt.title('finite difference solution')
plt.savefig("f.pdf")
