# provides 1D temperature profile in a 4-layer LMB
# can only be used with scalar values!

# -*- coding: utf-8 -*-
import numpy as np

# compute Rayleigh number according to Wikipedia
def Ra(h, beta, nu, rho, cp, k, dT):
    ''' Compute thermal Rayleigh number
        Arguments:
        - height of layer h
        - thermal expansion coefficient beta
        - kinematic viscosity nu
        - density rho
        - heat capacity cp
        - thermal conductivity k
        - temperature difference dT '''

    # gravity acceleration
    g = 9.81
    # thermal diffusivity
    alpha = k/(rho*cp)
    return g*beta*dT*h**3 / (nu*alpha)


# give temperature difference in anode, electrolyte, cathode
def deltaT(j, T_ref, lambda_a, lambda_sA, lambda_sC, lambda_c, sigma_sA, sigma_sC, h_a, h_sA, h_sC, h_c, n):
    ''' Return temperature difference in anode, electrolyte, cathode
        Arguments: 
        - current density, ref. temperature, 
        - thermal conductivity of anode, upper salt, lower salt, cathode
        - conductivities
        - height of the 4 layers
        - number of points
        Returns:
        - 4 temperature vectors
        - 4 coordinate vectors '''

    # geometry
    h1          = h_c                   # height interface cathode-salt
    h2          = h_c+h_sC;             # height interface salt-salt
    h3          = h_c+h_sC+h_sA;        # height interface salt-anode
    H           = h_c+h_sC+h_sA+h_a;    # height of the cell

    # z - For the Plot 
    z_c = np.linspace(h1-h1, h1, int(n*h_c/H)); # the int-term computes the number of points for each layer
    z_sC = np.linspace(h1, h2, int(n*h_sC/H));
    z_sA = np.linspace(h2, h3, int(n*h_sA/H));
    z_a = np.linspace(h3, H, int(n*h_a/H));

    # HEAT GENERATION
    qDot_sC =  j**2/sigma_sC;
    qDot_sA =  j**2/sigma_sA;

    # constants
    L1 = lambda_a*lambda_sA*lambda_sC;
    L2 = lambda_a*lambda_c*lambda_sA;
    L3 = lambda_a*lambda_c*lambda_sC;
    L4 = lambda_c*lambda_sA*lambda_sC;

    C5 = (
     h3*h_a*L4*qDot_sA - h1*h_sC*L1*qDot_sC + h2*h_sC*L2*qDot_sA
     + h1*h2*(L2*qDot_sC + L1*qDot_sA)
     + (h3**2-h2**2)*L3*qDot_sA/2 - (h1**2+h2**2)*L2*qDot_sC/2
     )\
     /\
     (lambda_sA* ( h_c*L1 + h_sC*L2 + h_sA*L3 + h_a*L4 ));
    C05 = (H*h3*lambda_c*lambda_sA*lambda_sC*qDot_sA - h1**2*lambda_a*lambda_c*lambda_sA*qDot_sC/2 + h1**2*lambda_a*lambda_sA*lambda_sC*qDot_sC - h1*h2*lambda_a*lambda_c*lambda_sA*qDot_sA + h1*h2*lambda_a*lambda_c*lambda_sA*qDot_sC + h1*h2*lambda_a*lambda_sA*lambda_sC*qDot_sA - h1*h2*lambda_a*lambda_sA*lambda_sC*qDot_sC + h2**2*lambda_a*lambda_c*lambda_sA*qDot_sA - h2**2*lambda_a*lambda_c*lambda_sA*qDot_sC/2 - h2**2*lambda_a*lambda_c*lambda_sC*qDot_sA/2 + h3**2*lambda_a*lambda_c*lambda_sC*qDot_sA/2 - h3**2*lambda_c*lambda_sA*lambda_sC*qDot_sA)/(lambda_sA*(H*lambda_c*lambda_sA*lambda_sC - h1*lambda_a*lambda_c*lambda_sA + h1*lambda_a*lambda_sA*lambda_sC + h2*lambda_a*lambda_c*lambda_sA - h2*lambda_a*lambda_c*lambda_sC + h3*lambda_a*lambda_c*lambda_sC - h3*lambda_c*lambda_sA*lambda_sC))
    C3 = (qDot_sC-qDot_sA) * h2 / lambda_sC + (lambda_sA/lambda_sC) * C5;
    C1 = -(qDot_sC/lambda_c) * h1 + (lambda_sC/lambda_c) * C3;
    C2 = T_ref
    C4 = (qDot_sC/2/lambda_sC) * h1**2 + (C1 - C3) * h1 + T_ref;
    C6 = (qDot_sA/2/lambda_sA - qDot_sC/2/lambda_sC) * h2**2 + (C3 - C5) * h2 + C4;
    C7 = -(qDot_sA/lambda_a) * h3 + (lambda_sA/lambda_a) * C5;
    C8 = T_ref - C7 * H;

    # temperature distribution
    def T_c(z) : # cathode
        TTT = (C1*z+C2);
        return TTT
    def T_sC(z) : # lower salt
        TTT = -qDot_sC/2/lambda_sC * z**2 + C3 * z + C4;
        return TTT
    def T_sA(z) : # upper salt
        TTT = -qDot_sA/2/lambda_sA * z**2 + C5 * z + C6;
        return TTT
    def T_a(z) : # anode
        TTT = (C7*z+C8);
        return TTT

    return T_a(z_a)-T_ref, T_sA(z_sA)-T_ref, T_sC(z_sC)-T_ref, T_c(z_c)-T_ref, z_a, z_sA, z_sC, z_c
