import numpy as np
import matplotlib.pyplot as plt
import pylab

# fonts and sizes
params = {'font.family' : 'Nimbus Roman', 'mathtext.fontset' : 'custom', 'mathtext.it' : 'STIXGeneral:italic', 'mathtext.fontset' : 'stix', 'mathtext.bf' : 'STIXGeneral:bold'}
pylab.rcParams.update(params)
plt.rcParams.update({'font.size': 16, 'lines.linewidth':2})
plt.rcParams['figure.figsize'] = [6, 4] # keep width 6, but you may change height (4)

# definition of HZDR-colours
hzdrBlue = '#005AA0'
blue80 = '#2473AD'
blue60 = '#6691C2'
blue40 = '#9BB3D7'
blue20 = '#CDD8EC'

hzdrOrange = '#F0781E'
orange80 = '#F39150'
orange60 = '#F7AF7C'
orange40 = '#FBCBA8'
orange20 = '#FDE6D4'

gray1 = '#515151'
gray2 = '#9C9C9C'
gray3 = '#B9B9B9'

# secondary colours (use only if really necessary)
hzdrDarkBlue = '#003E6E'
hzdrRed = '#D42D12'
hzdrYellow = '#E6AF11'
hzdrGreen = '#144D28'
hzdrLightGreen = '#A9B509'
hzdrLightBlue = '#00A2E0'


# formulae:
# voltage efficiency = discharge voltage / charge voltage = (OCV - ohmicLoss) / (OCV + ohmicLoss)
# voltage loss = j*h / sigma
# x-axis shows salt layer thickness of DISCHARGED cell


#CONSTANTS
sigma={'SaltDischarged':1.38,
       'SaltCharged':0.92} # S/cm

OCV=1.83 #V according to Xu

CurrentDensityRange=np.array([0.05,0.1,0.2,0.3,0.5,1])

saltThickness = np.arange(0,51,1) # in mm


# efficiency: dimension 1 = current; dimension 2 = salt thickness
efficiencyDischarged = np.empty((0,saltThickness.shape[0]))
efficiencyCharged = np.empty((0,saltThickness.shape[0]))

for CurrentDensity in CurrentDensityRange: # iterate through all current densities (A/cm2)
    VoltageEff_charged=[]	# empty list to save voltage efficency for each layer thickness
    VoltageEff_discharged=[]
    
    for thickness in saltThickness:
        # the salt layer thickness is assumed to be the same for a charged and dischared cell
        # the salt layer thickness for the discharged cell is used
        # this should be justivied in view of the many simplifications and the small volume change
        ohmicLoss_discharged=CurrentDensity*thickness*(1/10)/(sigma['SaltDischarged']) # 1/10 converts mm to cm (layer height)
        ohmicLoss_charged=CurrentDensity*thickness*(1/10)/(sigma['SaltCharged'])

        # these are the old, wrong formulae
        #VoltageEff_discharged.append(100*(OCV-ohmicLoss_discharged)/OCV)
        #VoltageEff_charged.append(100*(OCV-ohmicLoss_charged)/OCV)
        # these are corrected formulae
        VoltageEff_discharged.append(100*(OCV-ohmicLoss_discharged)/(OCV+ohmicLoss_discharged))
        VoltageEff_charged.append(100*(OCV-ohmicLoss_charged)/(OCV+ohmicLoss_charged))

    efficiencyDischarged = np.vstack((efficiencyDischarged, np.array(VoltageEff_discharged)))
    efficiencyCharged = np.vstack((efficiencyCharged, np.array(VoltageEff_charged)))
    
 
fig,ax=plt.subplots()
for i in range(CurrentDensityRange.size):
    label = str(CurrentDensityRange[i])+r'A/cm$^2$'
    ax.plot(saltThickness, efficiencyCharged[i], color=hzdrBlue, label=label)
    ax.plot(saltThickness, efficiencyDischarged[i], color=hzdrBlue)
    ax.fill_between(saltThickness, efficiencyCharged[i], efficiencyDischarged[i], alpha=0.15*i+0.1, color=hzdrBlue)
ax.plot([0,50],[80,80], color='k', linewidth=1, linestyle='--')

plt.xlabel('salt layer thickness (mm)')
plt.ylabel('voltage efficiency (%)')

plt.xlim(0,50)
plt.ylim(0,100)

ax.text(5,6,r'1$\,$A/cm$^{2}$', fontsize=16.)
ax.text(30.5,6,r'0.5$\,$A/cm$^{2}$', fontsize=16.)
ax.text(40,19,r'0.3$\,$A/cm$^{2}$', fontsize=16.)
ax.text(40,37,r'0.2$\,$A/cm$^{2}$', fontsize=16.)
ax.text(40,62,r'0.1$\,$A/cm$^{2}$', fontsize=16.)
ax.text(39,90,r'0.05$\,$A/cm$^{2}$', fontsize=16.)

fig.subplots_adjust(left=.13,bottom=.14,right=.96,top=.96,wspace=.2,hspace=.0)
fig.savefig("voltageEfficiency.pdf")
